@@ -183,7 +183,7 @@ def calc_blr(xsqlda):
     blr += [255, 76]    # [blr_end, blr_eoc]
 
     # x.sqlscale value shoud be negative, so b convert to range(0, 256)
-    return bytes(256 + b if b < 0 else b for b in blr)
+    return bs(256 + b if b < 0 else b for b in blr)
 
 def parse_select_items(buf, xsqlda, connection):
     index = 0
@@ -196,7 +196,6 @@ def parse_select_items(buf, xsqlda, connection):
             xsqlda[index-1] = XSQLVAR(connection.bytes_to_ustr
                                         if connection.use_unicode
                                         else connection.bytes_to_str)
-            xsqlda[index-1] = XSQLVAR(connection.bytes_to_str)
             i = i + 3 + l
         elif item == isc_info_sql_type:
             l = bytes_to_int(buf[i+1:i+3])
@@ -249,10 +248,10 @@ def parse_xsqlda(buf, connection, stmt_handle):
     stmt_type = None
     i = 0
     while i < len(buf):
-        if buf[i:i+3] == bytes([isc_info_sql_stmt_type,0x04,0x00]):
+        if buf[i:i+3] == bs([isc_info_sql_stmt_type,0x04,0x00]):
             stmt_type = bytes_to_int(buf[i+3:i+7])
             i += 7
-        elif buf[i:i+2] == bytes([isc_info_sql_select, isc_info_sql_describe_vars]):
+        elif buf[i:i+2] == bs([isc_info_sql_select, isc_info_sql_describe_vars]):
             i += 2
             l = bytes_to_int(buf[i:i+2])
             i += 2
@@ -261,11 +260,11 @@ def parse_xsqlda(buf, connection, stmt_handle):
             next_index = parse_select_items(buf[i+l:], xsqlda, connection)
             while next_index > 0:   # more describe vars
                 connection._op_info_sql(stmt_handle,
-                            bytes([isc_info_sql_sqlda_start, 2])
+                            bs([isc_info_sql_sqlda_start, 2])
                                 + int_to_bytes(next_index, 2)
                                 + INFO_SQL_SELECT_DESCRIBE_VARS)
                 (h, oid, buf) = connection._op_response()
-                assert buf[:2] == bytes([0x04,0x07])
+                assert buf[:2] == bs([0x04,0x07])
                 l = bytes_to_int(buf[2:4])
                 assert bytes_to_int(buf[4:4+l]) == col_len
                 next_index = parse_select_items(buf[4+l:], xsqlda, connection)
